@@ -16,6 +16,9 @@
 #' @param intercept `r lifecycle::badge("experimental")` Prior for the constant term by [set_intercept()].
 #' @param include_mean Add constant term (Default: `TRUE`) or not (`FALSE`)
 #' @param minnesota Apply cross-variable shrinkage structure (Minnesota-way). By default, `TRUE`.
+#' @param ggl If `TRUE` (default), use additional group shrinkage parameter for group structure.
+#' Otherwise, use group shrinkage parameter instead of global shirnkage parameter.
+#' Applies to HS, NG, and DL priors.
 #' @param save_init Save every record starting from the initial values (`TRUE`).
 #' By default, exclude the initial values in the record (`FALSE`), even when `num_burn = 0` and `thinning = 1`.
 #' If `num_burn > 0` or `thinning != 1`, this option is ignored.
@@ -86,6 +89,7 @@ var_bayes <- function(y,
                       intercept = set_intercept(),
                       include_mean = TRUE,
                       minnesota = TRUE,
+                      ggl = TRUE,
                       save_init = FALSE,
                       convergence = NULL,
                       verbose = FALSE,
@@ -122,9 +126,10 @@ var_bayes <- function(y,
     is.ssvsinput(bayes_spec) ||
     is.horseshoespec(bayes_spec) ||
     is.ngspec(bayes_spec) ||
-    is.dlspec(bayes_spec)
+    is.dlspec(bayes_spec) ||
+    is.gdpspec(bayes_spec)
   )) {
-    stop("Provide 'bvharspec', 'ssvsinput', 'horseshoespec', 'ngspec', or 'dlspec' for 'bayes_spec'.")
+    stop("Provide 'bvharspec', 'ssvsinput', 'horseshoespec', 'ngspec', 'dlspec', or 'gdpspec' for 'bayes_spec'.")
   }
   if (!is.covspec(cov_spec)) {
     stop("Provide 'covspec' for 'cov_spec'.")
@@ -170,13 +175,15 @@ var_bayes <- function(y,
     dim_data = dim_data,
     dim_design = num_alpha / dim_data,
     num_coef = num_alpha,
-    minnesota = ifelse(minnesota, "short", "no"),
+    minnesota = ifelse(minnesota, "longrun", "no"),
     include_mean = FALSE
   )
   grp_id <- unique(c(glob_idmat))
   if (minnesota) {
-    own_id <- 2
-    cross_id <- seq_len(p + 1)[-2]
+    # own_id <- 2
+    # cross_id <- seq_len(p + 1)[-2]
+    own_id <- seq(2, 2 * p, by = 2)
+    cross_id <- seq(1, 2 * p, by = 2)
   } else {
     own_id <- 1
     cross_id <- 2
@@ -195,10 +202,14 @@ var_bayes <- function(y,
     if (is.null(bayes_spec$delta)) {
       bayes_spec$delta <- rep(0, dim_data)
     }
+    if (length(bayes_spec$delta) == 1) {
+      bayes_spec$delta <- rep(bayes_spec$delta, dim_data)
+    }
     param_prior <- append(bayes_spec, list(p = p))
     if (bayes_spec$hierarchical) {
       param_prior$shape <- bayes_spec$lambda$param[1]
       param_prior$rate <- bayes_spec$lambda$param[2]
+      param_prior$grid_size <- bayes_spec$lambda$grid_size
       prior_nm <- "MN_Hierarchical"
       param_init <- lapply(
         param_init,
@@ -275,7 +286,9 @@ var_bayes <- function(y,
             coef_mixture = coef_mixture,
             coef_slab = init_coef_slab,
             chol_mixture = chol_mixture,
-            contem_slab = init_contem_slab
+            contem_slab = init_contem_slab,
+            coef_spike_scl = runif(1, 0, 1),
+            chol_spike_scl = runif(1, 0, 1)
           )
         )
       }
@@ -370,6 +383,34 @@ var_bayes <- function(y,
         )
       }
     )
+  } else if (prior_nm == "GDP") {
+    param_prior <- bayes_spec
+    param_init <- lapply(
+      param_init,
+      function(init) {
+        local_sparsity <- exp(runif(num_alpha, -1, 1))
+        group_rate <- exp(runif(num_grp, -1, 1))
+        contem_local_sparsity <- exp(runif(num_eta, -1, 1)) # sd = local * global
+        contem_local_rate <- exp(runif(num_eta, -1, 1))
+        coef_shape <- runif(1, 0, 1)
+        coef_rate <- runif(1, 0, 1)
+        contem_shape <- runif(1, 0, 1)
+        contem_rate <- runif(1, 0, 1)
+        append(
+          init,
+          list(
+            local_sparsity = local_sparsity,
+            group_rate = group_rate,
+            contem_local_sparsity = contem_local_sparsity,
+            contem_rate = contem_local_rate,
+            gamma_shape = coef_shape,
+            gamma_rate = coef_rate,
+            contem_gamma_shape = contem_shape,
+            contem_gamma_rate = contem_rate
+          )
+        )
+      }
+    )
   }
   prior_type <- switch(prior_nm,
     "Minnesota" = 1,
@@ -377,7 +418,8 @@ var_bayes <- function(y,
     "Horseshoe" = 3,
     "MN_Hierarchical" = 4,
     "NG" = 5,
-    "DL" = 6
+    "DL" = 6,
+    "GDP" = 7
   )
   if (num_thread > get_maxomp()) {
     warning("'num_thread' is greater than 'omp_get_max_threads()'. Check with bvhar:::get_maxomp(). Check OpenMP support of your machine with bvhar:::check_omp().")
@@ -408,27 +450,7 @@ var_bayes <- function(y,
         )
       }
     )
-    res <- estimate_var_sv(
-      num_chains = num_chains,
-      num_iter = num_iter,
-      num_burn = num_burn,
-      thin = thinning,
-      x = X0,
-      y = Y0,
-      param_sv = cov_spec[c("shape", "scale", "initial_mean", "initial_prec")],
-      param_prior = param_prior,
-      param_intercept = intercept[c("mean_non", "sd_non")],
-      param_init = param_init,
-      prior_type = prior_type,
-      grp_id = grp_id,
-      own_id = own_id,
-      cross_id = cross_id,
-      grp_mat = glob_idmat,
-      include_mean = include_mean,
-      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
-      display_progress = verbose,
-      nthreads = num_thread
-    )
+    param_cov <- cov_spec[c("shape", "scale", "initial_mean", "initial_prec")]
   } else {
     param_init <- lapply(
       param_init,
@@ -439,29 +461,30 @@ var_bayes <- function(y,
         )
       }
     )
-    res <- estimate_sur(
-      num_chains = num_chains,
-      num_iter = num_iter,
-      num_burn = num_burn,
-      thin = thinning,
-      x = X0,
-      y = Y0,
-      param_reg = cov_spec[c("shape", "scale")],
-      param_prior = param_prior,
-      param_intercept = intercept[c("mean_non", "sd_non")],
-      param_init = param_init,
-      prior_type = prior_type,
-      grp_id = grp_id,
-      own_id = own_id,
-      cross_id = cross_id,
-      grp_mat = glob_idmat,
-      include_mean = include_mean,
-      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
-      display_progress = verbose,
-      nthreads = num_thread
-    )
+    param_cov <- cov_spec[c("shape", "scale")]
   }
-  
+  res <- estimate_sur(
+    num_chains = num_chains,
+    num_iter = num_iter,
+    num_burn = num_burn,
+    thin = thinning,
+    x = X0,
+    y = Y0,
+    param_reg = param_cov,
+    param_prior = param_prior,
+    param_intercept = intercept[c("mean_non", "sd_non")],
+    param_init = param_init,
+    prior_type = prior_type,
+    ggl = ggl,
+    grp_id = grp_id,
+    own_id = own_id,
+    cross_id = cross_id,
+    grp_mat = glob_idmat,
+    include_mean = include_mean,
+    seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+    display_progress = verbose,
+    nthreads = num_thread
+  )
   res <- do.call(rbind, res)
   rec_names <- colnames(res)
   param_names <- gsub(pattern = "_record$", replacement = "", rec_names)
@@ -482,7 +505,7 @@ var_bayes <- function(y,
   res$sparse_coef <- matrix(colMeans(res$alpha_sparse_record), ncol = dim_data)
   if (include_mean) {
     res$coefficients <- rbind(res$coefficients, colMeans(res$c_record))
-    res$sparse_coef <- rbind(res$sparse_coef, colMeans(res$c_record))
+    res$sparse_coef <- rbind(res$sparse_coef, colMeans(res$c_sparse_record))
   }
   mat_lower <- matrix(0L, nrow = dim_data, ncol = dim_data)
   diag(mat_lower) <- rep(1L, dim_data)
@@ -558,7 +581,8 @@ var_bayes <- function(y,
   if (include_mean) {
     res$param <- bind_draws(
       res$param,
-      res$c_record
+      res$c_record,
+      res$c_sparse_record
     )
   }
   if (bayes_spec$prior == "SSVS") {
@@ -587,6 +611,8 @@ var_bayes <- function(y,
       res$lambda_record,
       res$tau_record
     )
+  } else if (bayes_spec$prior == "GDP") {
+    # 
   }
   res[rec_names] <- NULL
   res$param_names <- param_names
@@ -608,6 +634,7 @@ var_bayes <- function(y,
   #   res$prior_mean <- prior_mean
   #   res$prior_prec <- prior_prec
   # }
+  res$ggl <- ggl
   # variables------------
   res$df <- dim_design
   res$p <- p
@@ -644,6 +671,8 @@ var_bayes <- function(y,
     class(res) <- c(class(res), "ngmod")
   } else if (bayes_spec$prior == "DL") {
     class(res) <- c(class(res), "dlmod")
+  } else if (bayes_spec$prior == "GDP") {
+    class(res) <- c(class(res), "gdpmod")
   }
   res
 }
